"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.APIReferenceModel = void 0;
function _semanticsEntities() {
  const data = require("@teambit/semantics.entities.semantic-schema");
  _semanticsEntities = function () {
    return data;
  };
  return data;
}
function _componentId() {
  const data = require("@teambit/component-id");
  _componentId = function () {
    return data;
  };
  return data;
}
function _lodash() {
  const data = _interopRequireDefault(require("lodash.compact"));
  _lodash = function () {
    return data;
  };
  return data;
}
function _lodash2() {
  const data = _interopRequireDefault(require("lodash.head"));
  _lodash2 = function () {
    return data;
  };
  return data;
}
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
class APIReferenceModel {
  internalAPIKey(schema, internalFilePath) {
    return this.generateInternalAPIKey(internalFilePath || schema.location.filePath, schema.name);
  }
  generateInternalAPIKey(filePath, name = '') {
    return `${filePath}/${name}`;
  }
  taggedAPINodes = [];
  constructor(_api, _renderers) {
    this._api = _api;
    this.componentId = _api.componentId;
    this.apiNodes = this.mapToAPINode(_api, _renderers, this.componentId);
    this.apiByType = this.groupByType(this.apiNodes);
    this.apiByName = this.groupByName(this.apiNodes);
    this.taggedAPINodes = this.mapTaggedAPINode(_api, _renderers, this.componentId);
  }
  mapTaggedAPINode(api, renderers, componentId) {
    const {
      taggedModuleExports
    } = api;
    const defaultRenderers = renderers.filter(renderer => renderer.default);
    const nonDefaultRenderers = renderers.filter(renderer => !renderer.default);
    return taggedModuleExports.map(schemaNode => ({
      componentId,
      api: schemaNode,
      exported: true,
      renderer: nonDefaultRenderers.find(renderer => renderer.predicate(schemaNode)) || defaultRenderers.find(renderer => renderer.predicate(schemaNode))
    })).filter(schemaNode => schemaNode.renderer);
  }
  getByType(type) {
    return this.apiByType.get(type) ?? [];
  }
  getByName(node, internalFilePath) {
    if (!node.name) return undefined;
    return this.apiByName.get(node.name) ?? this.apiByName.get(this.internalAPIKey(node, internalFilePath));
  }
  groupByType(apiNodes) {
    return apiNodes.reduce((accum, next) => {
      const existing = accum.get(next.renderer.nodeType) || [];
      accum.set(next.renderer.nodeType, existing.concat(next));
      return accum;
    }, new Map());
  }
  groupByName(apiNodes) {
    return apiNodes.reduce((accum, next) => {
      const name = next.alias || next.api.name;
      if (!name) return accum;
      const key = next.exported ? name : this.internalAPIKey(next.api);
      accum.set(key, next);
      return accum;
    }, new Map());
  }
  isInternalTypeReference(node) {
    return node.__schema === 'TypeRefSchema' && node.isInternalReference();
  }
  mapToAPINode(api, renderers, componentId) {
    const {
      exports: exportedSchemaNodes
    } = api.module;
    const collectedInternals = [];
    exportedSchemaNodes.forEach(exportedNode => {
      const targetNode = _semanticsEntities().ExportSchema.isExportSchema(exportedNode) ? exportedNode.exportNode : exportedNode;
      const allDescendants = targetNode.getAllNodesRecursively();
      allDescendants.forEach(descendant => {
        if (descendant !== targetNode && this.isInternalTypeReference(descendant)) {
          collectedInternals.push(descendant);
        }
      });
    });
    const dedupedInternalsMap = new Map();
    for (const node of collectedInternals) {
      const key = this.internalAPIKey(node);
      if (!dedupedInternalsMap.has(key)) {
        dedupedInternalsMap.set(key, node);
      }
    }
    const internalSchemaNodes = Array.from(dedupedInternalsMap.values());
    const defaultRenderers = renderers.filter(renderer => renderer.default);
    const nonDefaultRenderers = renderers.filter(renderer => !renderer.default);
    const exportedAPINodes = exportedSchemaNodes.map(schemaNode => {
      const targetNode = _semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.exportNode : schemaNode;
      return {
        componentId,
        api: targetNode,
        alias: _semanticsEntities().ExportSchema.isExportSchema(schemaNode) ? schemaNode.alias : undefined,
        exported: true,
        renderer: nonDefaultRenderers.find(renderer => renderer.predicate(targetNode)) || defaultRenderers.find(renderer => renderer.predicate(targetNode))
      };
    });
    const internalAPINodes = internalSchemaNodes.map(schemaNode => {
      const internalSchemaNode = (0, _lodash2().default)((0, _lodash().default)(api.internals.flatMap(i => i.exports).map(e => e.findNode(s => s.name === schemaNode.name && s.__schema !== 'TypeRefSchema')))) || schemaNode;
      return {
        componentId,
        api: internalSchemaNode,
        exported: false,
        renderer: nonDefaultRenderers.find(renderer => renderer.predicate(internalSchemaNode)) || defaultRenderers.find(renderer => renderer.predicate(internalSchemaNode))
      };
    });
    return exportedAPINodes.concat(internalAPINodes).filter(node => node.renderer);
  }
  static from(result, renderers = []) {
    try {
      const apiSchema = _semanticsEntities().APISchema.fromObject(result.getHost.getSchema);
      return new APIReferenceModel(apiSchema, renderers);
    } catch {
      return new APIReferenceModel(_semanticsEntities().APISchema.empty(_componentId().ComponentID.fromObject(result.getHost.getSchema.componentId)), renderers);
    }
  }
}
exports.APIReferenceModel = APIReferenceModel;

//# sourceMappingURL=api-reference-model.js.map